
// apb_sd_host_demo
module apb_sd_host_demo
(
  JTAG_7,           // SWDIO
  JTAG_9,           // SWDCLK
  UART0RXD,         // UART0
  UART0TXD,
  SD_CLK,           // APB SD Host
  SD_CMD,
  SD_DATA,
  SD_SWITCH,
  SD_DETECT,
  FLASH_SPI_CSN,    // SPI Flash Memory
  FLASH_SPI_MISO,
  FLASH_SPI_MOSI,
  FLASH_SPI_CLK,
  HCLK,
  hwRstn
);

// JTAG
inout JTAG_7;   // SWDIO
inout JTAG_9;   // SWDCLK

// UART0
input UART0RXD;
output UART0TXD;

// APB SD Host
output          SD_CLK;
inout           SD_CMD;
inout [3:0]     SD_DATA;
input           SD_SWITCH;
output          SD_DETECT;

input HCLK;     // Clock
input hwRstn;   // Reset, active-low

// SPI Flash Memory
inout FLASH_SPI_CSN;
inout FLASH_SPI_MISO;
inout FLASH_SPI_MOSI;
inout FLASH_SPI_CLK;


// APB Master [1]
wire            APB1PCLK;
wire            APB1PRESET;
wire            APB1PENABLE;
wire [31:0]     APB1PADDR;
wire            APB1PWRITE;
wire [31:0]     APB1PWDATA;
wire [3:0]      APB1PSTRB;
wire [2:0]      APB1PPROT;
wire            APB1PSEL;
wire [31:0]     APB1PRDATA;
wire            APB1PREADY;
wire            APB1PSLVERR;

// APB SD Host
wire        sd_clk_o_pad;
wire        sd_cmd_dat_i;
wire        sd_cmd_out_o;
wire        sd_cmd_oe_o;
wire [3:0]  sd_dat_dat_i;
wire [3:0]  sd_dat_out_o;
wire        sd_dat_oe_o;
wire        int_cmd;
wire        int_data;


assign SD_CLK = sd_clk_o_pad;
assign SD_DETECT = SD_SWITCH;


// APB SD Host inout port
gw_inout_pad sd_cmd (.O(sd_cmd_dat_i),      .I(sd_cmd_out_o),      .IO(SD_CMD),       .E(sd_cmd_oe_o),   .PU(1'b0), .PD(1'b0));
gw_inout_pad sd_d0	(.O(sd_dat_dat_i[0]),   .I(sd_dat_out_o[0]),   .IO(SD_DATA[0]),   .E(sd_dat_oe_o),   .PU(1'b0), .PD(1'b0));
gw_inout_pad sd_d1	(.O(sd_dat_dat_i[1]),   .I(sd_dat_out_o[1]),   .IO(SD_DATA[1]),   .E(sd_dat_oe_o),   .PU(1'b0), .PD(1'b0));
gw_inout_pad sd_d2	(.O(sd_dat_dat_i[2]),   .I(sd_dat_out_o[2]),   .IO(SD_DATA[2]),   .E(sd_dat_oe_o),   .PU(1'b0), .PD(1'b0));
gw_inout_pad sd_d3	(.O(sd_dat_dat_i[3]),   .I(sd_dat_out_o[3]),   .IO(SD_DATA[3]),   .E(sd_dat_oe_o),   .PU(1'b0), .PD(1'b0));


// APB_SD_Host_Top instantiation
APB_SD_Host_Top u_APB_SD_Host_Top
(
    .pclk(APB1PCLK),
    .presetn(APB1PRESET),
    .psel(APB1PSEL),
    .penable(APB1PENABLE),
    .pwrite(APB1PWRITE),
    .paddr(APB1PADDR[7:0]),
    .pstrb(APB1PSTRB),
    .pwdata(APB1PWDATA),
    .prdata(APB1PRDATA),
    .pready(APB1PREADY),
    .pslverr(APB1PSLVERR),
    .sd_cmd_dat_i(sd_cmd_dat_i), 
    .sd_cmd_out_o(sd_cmd_out_o), 
    .sd_cmd_oe_o(sd_cmd_oe_o), 
    .sd_dat_dat_i(sd_dat_dat_i), 
    .sd_dat_out_o(sd_dat_out_o), 
    .sd_dat_oe_o(sd_dat_oe_o), 
    .sd_clk_o_pad(sd_clk_o_pad),
    .sd_clk_i_pad(APB1PCLK),
    .int_cmd(int_cmd), 
    .int_data(int_data)
);


//Gowin_EMPU_M1 instantiation
Gowin_EMPU_M1_Top u_Gowin_EMPU_M1_Top
(
    .LOCKUP(),
    .HALTED(),
    .EXTINT({2'b0, int_data, int_cmd}),
    .JTAG_7(JTAG_7),
    .JTAG_9(JTAG_9),
    .UART0RXD(UART0RXD),
    .UART0TXD(UART0TXD),
    .FLASH_SPI_HOLDN(),
    .FLASH_SPI_CSN(FLASH_SPI_CSN),
    .FLASH_SPI_MISO(FLASH_SPI_MISO),
    .FLASH_SPI_MOSI(FLASH_SPI_MOSI),
    .FLASH_SPI_WPN(),
    .FLASH_SPI_CLK(FLASH_SPI_CLK),
    .APB1PADDR(APB1PADDR),
    .APB1PENABLE(APB1PENABLE),
    .APB1PWRITE(APB1PWRITE),
    .APB1PSTRB(APB1PSTRB),
    .APB1PPROT(APB1PPROT),
    .APB1PWDATA(APB1PWDATA),
    .APB1PSEL(APB1PSEL),
    .APB1PRDATA(APB1PRDATA),
    .APB1PREADY(APB1PREADY),
    .APB1PSLVERR(APB1PSLVERR),
    .APB1PCLK(APB1PCLK),
    .APB1PRESET(APB1PRESET),
    .HCLK(HCLK),
    .hwRstn(hwRstn)
);

endmodule




// gw_inout_pad
module gw_inout_pad
(
    O,
    I,
    IO,
    E,
    PU,
    PD
);

output  O;
input   I;
inout   IO;
input   E;
input   PU;
input   PD;

assign  IO	= E ? I : 1'bz;
assign  O	= IO;
assign  (pull0, pull1) IO = PU ? 1'b1 :
                            PD ? 1'b0 : 1'bz;

endmodule