/*
 * ******************************************************************************************
 *
 * File     : sd.h
 * Author   : GOWIN
 * Function	: Defines of APB SD Host for FATFS
 *
 * ******************************************************************************************
 */

#ifndef __SD_H__
#define __SD_H__


#ifdef __cplusplus
extern "C" {
#endif


/* Includes ------------------------------------------------------------------*/
#include <stdint.h>


/* Defines -------------------------------------------------------------------*/

/**
  * APB SD Host card specific data: csd register
  */
typedef struct
{
  uint8_t  csd_struct;                           /*!< csd structure */
  uint8_t  spec_version;                         /*!< system specification version */
  uint8_t  reserved1;                            /*!< reserved */
  uint8_t  taac;                                 /*!< data read access-time 1 */
  uint8_t  nsac;                                 /*!< data read access-time 2 in clk cycles */
  uint8_t  max_bus_clk_freq;                     /*!< max. bus clock frequency */
  uint16_t card_cmd_classes;                     /*!< card command classes */
  uint8_t  max_read_blk_length;                  /*!< max. read data block length */
  uint8_t  part_blk_read;                        /*!< partial blocks for read allowed */
  uint8_t  write_blk_misalign;                   /*!< write block misalignment */
  uint8_t  read_blk_misalign;                    /*!< read block misalignment */
  uint8_t  dsr_implemented;                      /*!< dsr implemented */
  uint8_t  reserved2;                            /*!< reserved */
  uint32_t device_size;                          /*!< device size */
  uint8_t  max_read_current_vdd_min;             /*!< max. read current @ vdd min */
  uint8_t  max_read_current_vdd_max;             /*!< max. read current @ vdd max */
  uint8_t  max_write_current_vdd_min;            /*!< max. write current @ vdd min */
  uint8_t  max_write_current_vdd_max;            /*!< max. write current @ vdd max */
  uint8_t  device_size_mult;                     /*!< device size multiplier */
  uint8_t  erase_group_size;                     /*!< erase group size */
  uint8_t  erase_group_size_mult;                /*!< erase group size multiplier */
  uint8_t  write_protect_group_size;             /*!< write protect group size */
  uint8_t  write_protect_group_enable;           /*!< write protect group enable */
  uint8_t  manufacturer_default_ecc;             /*!< manufacturer default ecc */
  uint8_t  write_speed_factor;                   /*!< write speed factor */
  uint8_t  max_write_blk_length;                 /*!< max. write data block length */
  uint8_t  part_blk_write;                       /*!< partial blocks for write allowed */
  uint8_t  reserved3;                            /*!< reserded */
  uint8_t  content_protect_app;                  /*!< content protection application */
  uint8_t  file_format_group;                    /*!< file format group */
  uint8_t  copy_flag;                            /*!< copy flag (otp) */
  uint8_t  permanent_write_protect;              /*!< permanent write protection */
  uint8_t  temp_write_protect;                   /*!< temporary write protection */
  uint8_t  file_formart;                         /*!< file format */
  uint8_t  ecc_code;                             /*!< ecc code */
  uint8_t  csd_crc;                              /*!< csd crc */
  uint8_t  reserved4;                            /*!< always */
} sd_csd_struct_type;


/**
  * sd card information
  */
typedef struct
{
  sd_csd_struct_type sd_csd_reg;
  uint64_t card_capacity;
  uint32_t card_blk_size;
} sd_card_info_struct_type;


/* Functions: */
uint8_t sd_init(void);
uint32_t sd_write_disk(const uint8_t *buf, uint32_t sector, uint8_t cnt);
uint32_t sd_read_disk(uint8_t *buf, uint32_t sector, uint8_t cnt);


#ifdef __cplusplus
}
#endif


#endif	/* __SD_H__ */
