/*
 * ******************************************************************************************
 *
 * File     : apb_sd_host_driver.c
 * Author   : GOWIN
 * Function	: Driver definition of APB SD Host
 *
 * ******************************************************************************************
 */


// Includes ---------------------------------------------------------------------------------
#include "apb_sd_host_driver.h"


// Defines ----------------------------------------------------------------------------------

// Software reset
void apb_sd_host_reset(APB_SD_Host_RegDef* sd_host, sd_reset_type reset_state)
{
	if(reset_state == SD_RESET)
	{
		sd_host->reset = 1;
		sd_host->reset = 0;
	}
	else
	{
		sd_host->reset = 0;
	}
}

// Configure clock
void apb_sd_host_clock_config(APB_SD_Host_RegDef* sd_host, sd_clk_div_type clk_div)
{
	sd_host->clock_divider = clk_div;
}

// Configure bus width
void apb_sd_host_bus_width_config(APB_SD_Host_RegDef* sd_host, sd_bus_width_type width)
{
	sd_host->control = width;
}

// Configure command
void apb_sd_host_command_config(APB_SD_Host_RegDef* sd_host, sd_command_struct_type *command_struct)
{
	sd_host->cmd_timeout = command_struct->timeout;

	sd_host->command = (command_struct->rsp_type)                | 
	                   ((command_struct->wait_type) << 2)        |
			               (command_struct->crc << 3)                | 
	                   (command_struct->index << 4)              |
					           (command_struct->transfer_direction << 5) | 
	                   ((command_struct->cmd_index) << 8);

	sd_host->argument = command_struct->argument;
}

// Get response
unsigned int apb_sd_host_response_get(APB_SD_Host_RegDef* sd_host, sd_rsp_index_type reg_index)
{
  unsigned int response_value = 0;

  switch(reg_index)
  {
    case SD_RSP1_INDEX:
      response_value = sd_host->response0;
      break;
    case SD_RSP2_INDEX:
      response_value = sd_host->response1;
      break;
    case SD_RSP3_INDEX:
      response_value = sd_host->response2;
      break;
    case SD_RSP4_INDEX:
      response_value = sd_host->response3;
      break;
    default:
			break;
  }

  return response_value;
}

// Configure data
void apb_sd_host_data_config(APB_SD_Host_RegDef* sd_host, sd_data_struct_type *data_struct)
{
	sd_host->data_timeout = data_struct->timeout;

	sd_host->blkock_count = data_struct->block_cnt;
	sd_host->blkock_size = data_struct->block_size;
}

// Write data
void apb_sd_host_data_write(APB_SD_Host_RegDef* sd_host, unsigned int *w_data)
{
	sd_host->tx_data = *w_data;
}

// Read data
void apb_sd_host_data_read(APB_SD_Host_RegDef* sd_host, unsigned int *r_data)
{
	*r_data = sd_host->rx_data;
}

// Enable interrupt
void apb_sd_host_interrupt_enable(APB_SD_Host_RegDef* sd_host, sd_interrupt_type type, unsigned char int_opt,  unsigned char new_state)
{
	if(type == SD_CMD_INT)
	{
		/* enable interrupt */
		if(1 == new_state)
		{
		  sd_host->cmd_event_enable |= int_opt;
		}
		/* disable interrupt */
		else
		{
		  sd_host->cmd_event_enable &= ~(int_opt);
		}
	}
	else if(type == SD_DATA_INT)
	{
		/* enable interrupt */
		if(1 == new_state)
		{
			sd_host->data_event_enable |= int_opt;
		}
		/* disable interrupt */
		else
		{
			sd_host->data_event_enable &= ~(int_opt);
		}
	}
}

// Get interrupt status
unsigned char apb_sd_host_interrupt_status_get(APB_SD_Host_RegDef* sd_host, sd_interrupt_type type, unsigned char flag)
{
	unsigned char status = 0;

	if(type == SD_CMD_INT)
	{
		if((sd_host->cmd_event_enable & flag) && (sd_host->cmd_event_status & flag))
		{
			status = 1;
		}
	}
	else if(type == SD_DATA_INT)
	{
		if((sd_host->data_event_enable & flag) && (sd_host->data_event_status & flag))
		{
			status = 1;
		}
	}

	return status;
}

// Clear interrupt status
void apb_sd_host_interrupt_status_clear(APB_SD_Host_RegDef* sd_host, sd_interrupt_type type, unsigned int flag)
{
	if(type == SD_CMD_INT)
	{
		sd_host->cmd_event_status = flag;
	}
	else if(type == SD_DATA_INT)
	{
		sd_host->data_event_status = flag;
	}
}

// Get FIFO status
unsigned char apb_sd_host_fifo_status_get(APB_SD_Host_RegDef* sd_host, sd_fifo_type fifo_type)
{
	unsigned char status = 0;

	if(fifo_type == SD_W_FIFO_FULL)
	{
		status = (sd_host->fifo_status & (1 << 2));
	}
	else if(fifo_type == SD_W_FIFO_EMPTY)
	{
		status = (sd_host->fifo_status & (1 << 3));
	}
	else if(fifo_type == SD_R_FIFO_FULL)
	{
		status = (sd_host->fifo_status & (1 << 0));
	}
	else if(fifo_type == SD_R_FIFO_EMPTY)
	{
		status = (sd_host->fifo_status & (1 << 1));
	}

	return status;
}
