/*
 * ******************************************************************************************
 *
 * File		  : main.c
 * Author	  : GOWIN
 * Function	: APB SD Host fatfs demo
 *
 * ******************************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "uart.h"
#include <stdio.h>
#include "delay.h"

#include "ff.h"


/* Defines -------------------------------------------------------------------*/

FATFS fs;
FIL file;
BYTE work[FF_MAX_SS];

uint8_t buffer_compare(uint8_t* pbuffer1, uint8_t* pbuffer2, uint16_t buffer_length)
{
  while(buffer_length--)
  {
    if(*pbuffer1 != *pbuffer2)
    {
      return 0;
    }
    pbuffer1++;
    pbuffer2++;
  }
	
  return 1;
}

//Application entry function
int main(void)
{
  SystemInit();  								//Initializes system clock
  uart_init(UART0,							//Initializes UART0
	          115200,							//Baudrate
	          1,									//Tx
	          1,									//Rx
	          0,									//Tx interrupt
	          0,									//Rx interrupt
	          0,									//Tx overflow interrupt
	          0);									//Rx overflow interrupt
	delay_init();									//Initializes delay function
	
	FRESULT ret;
	char filename[] = "1:/test1.txt";
	const char wbuf[] = "This is my file for test fatfs!\r\n";
	char rbuf[50];
	UINT bytes_written = 0;
	UINT bytes_read = 0;
	
	printf("\r\nIt's an APB SD Host fatfs demo.\r\n\r\n");
	
	printf("Start test fatfs R0.15a..\r\n");

	ret = f_mount(&fs, "1:", 1);
	if(ret)
	{
		printf("File system mount error:%d.\r\n", ret);

		if(ret == FR_NO_FILESYSTEM)
			{
		  printf("Create fatfs...\r\n");

		  ret = f_mkfs("1:", 0, work, sizeof(work));
		  if(ret)
			{
				printf("Create fatfs error:%d.\r\n", ret);
		  }
		  else
			{
				printf("Creates fatfs ok.\r\n");
		  }

		  ret = f_mount(NULL, "1:", 1);
		  ret = f_mount(&fs, "1:", 1);
		  if(ret)
			{
				printf("File system mount error:%d.\r\n", ret);
		  }
		  else
			{
				printf("File system mount ok.\r\n");
		  }
		}
		else
		{
			printf("ERROR.\r\n");
			return 0;
		}
	}
	else
	{
		printf("File system mount ok.\r\n");
	}

	ret = f_open(&file, filename, FA_READ | FA_WRITE | FA_CREATE_ALWAYS);
	if(ret)
	{
		printf("Open file error:%d.\r\n", ret);
	}
	else
	{
		printf("Open file ok.\r\n");
	}

	ret = f_write(&file, wbuf, sizeof(wbuf), &bytes_written);
	if(ret)
	{
		printf("Write file err:%d.\r\n", ret);
	}
	else
	{
		printf("Write file ok, byte:%u.\r\n", bytes_written);
	}

	f_lseek(&file, 0);
	ret = f_read(&file, rbuf, sizeof(rbuf), &bytes_read);
	if(ret)
	{
		printf("Read file error:%d.\r\n", ret);
	}
	else
	{
		printf("Read file ok, byte:%u.\r\n", bytes_read);
	}

	ret = f_close(&file);
	if(ret)
	{
		printf("Close file error:%d.\r\n", ret);
	}
	else
	{
		printf("Close file ok.\r\n");
	}

	ret = f_mount(NULL, "1:", 1);

	if(1 == buffer_compare((uint8_t*)rbuf, (uint8_t*)wbuf, sizeof(wbuf)))
	{
		printf("Read and write file data test ok.\r\n");
	}
	else
	{
		printf("Read and write file data test fail.\r\n");
	}

	return 0;
}
