/*
 * ******************************************************************************************
 *
 * File     : sd.c
 * Author   : GOWIN
 * Function	: Defines of APB SD Host for FATFS
 *
 * ******************************************************************************************
 */

// Includes ---------------------------------------------------------------------------------
#include "sd.h"
#include "apb_sd_host_driver.h"
#include "apb_sd_host_dev.h"
#include "delay.h"

#include <stdio.h>


/* Defines -------------------------------------------------------------------*/

#define CMDTIMEOUT		0x7FFF
#define DATATIMEOUT		0x7FFFFF

static uint32_t rca = 0, csd_table[4];	/* rca, csd */
static uint32_t SDIO_HIGH_CAPACITY_SD_CARD = 0;

sd_command_struct_type sd_command_init_struct;
sd_data_struct_type sd_data_init_struct;
sd_card_info_struct_type sd_card_info;
sd_card_info_struct_type *card_info = &sd_card_info;


//SD card identification
void sd_identification(void)
{
	uint8_t tmp = 0;

	/* send cmd2, get cid register */
	sd_command_init_struct.rsp_type = SD_RESPONSE_LONG;
	sd_command_init_struct.wait_type = SD_WAIT_FOR_BUSY;
	sd_command_init_struct.cmd_index = SD_CMD_ALL_SEND_CID;
	sd_command_init_struct.argument = 0x0;
	apb_sd_host_command_config(APB_SD1, &sd_command_init_struct);

	delay_ms(1000);

  /* send cmd3, get rca */
	sd_command_init_struct.rsp_type = SD_RESPONSE_SHORT;
	sd_command_init_struct.wait_type = SD_WAIT_FOR_BUSY;
	sd_command_init_struct.cmd_index = SD_CMD_SET_REL_ADDR;
	sd_command_init_struct.argument = 0x100;
	apb_sd_host_command_config(APB_SD1, &sd_command_init_struct);

	delay_ms(1000);

	rca = apb_sd_host_response_get(APB_SD1, SD_RSP1_INDEX);

	/* send cmd9, get csd */
	sd_command_init_struct.rsp_type = SD_RESPONSE_LONG;
	sd_command_init_struct.wait_type = SD_WAIT_FOR_BUSY;
	sd_command_init_struct.cmd_index = SD_CMD_SEND_CSD;
	sd_command_init_struct.argument = rca & 0xFFFF0000;
	apb_sd_host_command_config(APB_SD1, &sd_command_init_struct);

	delay_ms(1000);

	csd_table[0] = apb_sd_host_response_get(APB_SD1, SD_RSP1_INDEX);
	csd_table[1] = apb_sd_host_response_get(APB_SD1, SD_RSP2_INDEX);
	csd_table[2] = apb_sd_host_response_get(APB_SD1, SD_RSP3_INDEX);
	csd_table[3] = apb_sd_host_response_get(APB_SD1, SD_RSP4_INDEX);

	if(SDIO_HIGH_CAPACITY_SD_CARD)
	{
		/* byte 7 */
		tmp = (uint8_t)(csd_table[1] & 0x000000FF);
		card_info->sd_csd_reg.device_size = (tmp & 0x3F) << 16;

		/* byte 8 */
		tmp = (uint8_t)((csd_table[2] & 0xFF000000) >> 24);

		card_info->sd_csd_reg.device_size |= (tmp << 8);

		/* byte 9 */
		tmp = (uint8_t)((csd_table[2] & 0x00FF0000) >> 16);

		card_info->sd_csd_reg.device_size |= (tmp);

		card_info->card_capacity = (uint64_t)(card_info->sd_csd_reg.device_size + 1) * 512 * 1024;
		card_info->card_blk_size = 512;

		printf("SD card capacity: %5.3fGB\r\n", (float)((card_info->sd_csd_reg.device_size + 1) * 512)/1024/1024);
	}
	else
	{
		/* byte 5 */
		tmp = (uint8_t)((csd_table[1] & 0x00FF0000) >> 16);
		card_info->sd_csd_reg.max_read_blk_length = tmp & 0x0F;

		/* byte 6 */
		tmp = (uint8_t)((csd_table[1] & 0x0000FF00) >> 8);
		card_info->sd_csd_reg.device_size = (tmp & 0x03) << 10;

		/* byte 7 */
		tmp = (uint8_t)(csd_table[1] & 0x000000FF);
		card_info->sd_csd_reg.device_size |= (tmp) << 2;

		/* byte 8 */
		tmp = (uint8_t)((csd_table[2] & 0xFF000000) >> 24);
		card_info->sd_csd_reg.device_size |= (tmp & 0xC0) >> 6;

		card_info->sd_csd_reg.max_read_current_vdd_min = (tmp & 0x38) >> 3;
		card_info->sd_csd_reg.max_read_current_vdd_max = (tmp & 0x07);

		/* byte 9 */
		tmp = (uint8_t)((csd_table[2] & 0x00FF0000) >> 16);
		card_info->sd_csd_reg.max_write_current_vdd_min = (tmp & 0xE0) >> 5;
		card_info->sd_csd_reg.max_write_current_vdd_max = (tmp & 0x1C) >> 2;
		card_info->sd_csd_reg.device_size_mult = (tmp & 0x03) << 1;
		
		/* byte 10 */
		tmp = (uint8_t)((csd_table[2] & 0x0000FF00) >> 8);
		card_info->sd_csd_reg.device_size_mult |= (tmp & 0x80) >> 7;

		card_info->card_capacity = (card_info->sd_csd_reg.device_size + 1) ;
		card_info->card_capacity *= (1 << (card_info->sd_csd_reg.device_size_mult + 2));
		card_info->card_blk_size = 1 << (card_info->sd_csd_reg.max_read_blk_length);
		card_info->card_capacity *= card_info->card_blk_size;

		printf("SD card capacity: %4.3fGB\r\n", (float)((card_info->sd_csd_reg.device_size + 1) *
				                                    (1 << (card_info->sd_csd_reg.device_size_mult + 2)) *
											                      (1 << (card_info->sd_csd_reg.max_read_blk_length)))/1024/1024/1024);
	}

	/* send cmd7, select card */
	sd_command_init_struct.rsp_type = SD_RESPONSE_SHORT;
	sd_command_init_struct.wait_type = SD_WAIT_FOR_BUSY;
	sd_command_init_struct.cmd_index = SD_CMD_SEL_DESEL_CARD;
	sd_command_init_struct.argument = rca & 0xFFFF0000;
	apb_sd_host_command_config(APB_SD1, &sd_command_init_struct);

	delay_ms(1000);
}

//Initializes SD card
uint8_t sd_init(void)
{
	uint8_t retry = 10, status;

	/* Set clock divider */
	apb_sd_host_clock_config(APB_SD1, SD_CLK_DIV18);

	/* Set data width */
	apb_sd_host_bus_width_config(APB_SD1, SD_BUS_WIDTH_D4);

	/* send cmd0, get in idle stage */
	sd_command_init_struct.timeout = CMDTIMEOUT;
	sd_command_init_struct.rsp_type = SD_RESPONSE_NO;
	sd_command_init_struct.wait_type = SD_WAIT_FOR_NO;
	sd_command_init_struct.cmd_index = SD_CMD_GO_IDLE_STATE;
	sd_command_init_struct.argument = 0x0;
	apb_sd_host_command_config(APB_SD1, &sd_command_init_struct);

	/* send cmd8, check card interface feature */
	do
	{
		sd_command_init_struct.rsp_type = SD_RESPONSE_SHORT;
		sd_command_init_struct.wait_type = SD_WAIT_FOR_BUSY;
		sd_command_init_struct.cmd_index = SD_CMD_HS_SEND_IF_COND;
		sd_command_init_struct.argument = SD_CHECK_PATTERN;
		apb_sd_host_command_config(APB_SD1, &sd_command_init_struct);

		delay_ms(1000);

	}while(((apb_sd_host_response_get(APB_SD1, SD_RSP1_INDEX) >> 8) != 0x000001) && retry--);

	/* send cmd55, check sd version */
	retry = 10;
	
	do
	{
		/* send cmd55 before acmd41 */
		sd_command_init_struct.rsp_type = SD_RESPONSE_SHORT;
		sd_command_init_struct.wait_type = SD_WAIT_FOR_BUSY;
		sd_command_init_struct.cmd_index = SD_CMD_APP_CMD;
		sd_command_init_struct.argument = 0x0;
		apb_sd_host_command_config(APB_SD1, &sd_command_init_struct);

		delay_ms(1000);

		sd_command_init_struct.wait_type = SD_WAIT_FOR_NO;
		sd_command_init_struct.cmd_index = SD_ACMD_SD_SEND_OP_COND;
		sd_command_init_struct.argument = 0x40ff8000;
		apb_sd_host_command_config(APB_SD1, &sd_command_init_struct);

		delay_ms(1000);
	}while(!((apb_sd_host_response_get(APB_SD1, SD_RSP1_INDEX) >> 24) & 0x80) && retry--);

	/* check sd card power on stage is success or not */
	if((apb_sd_host_response_get(APB_SD1, SD_RSP1_INDEX) >> 24) & 0x80)
	{
		status = 1;
		SDIO_HIGH_CAPACITY_SD_CARD = 0;

		/* check sd card is SDHC or not */
		if((apb_sd_host_response_get(APB_SD1, SD_RSP1_INDEX) >> 24) & 0x40)
		{
			status = 2;
			SDIO_HIGH_CAPACITY_SD_CARD = 1;
		}
	}

	sd_identification();

	/* set data width */
	sd_command_init_struct.rsp_type = SD_RESPONSE_SHORT;
	sd_command_init_struct.wait_type = SD_WAIT_FOR_BUSY;
	sd_command_init_struct.cmd_index = SD_CMD_APP_CMD;
	sd_command_init_struct.argument = rca & 0xFFFF0000;
	apb_sd_host_command_config(APB_SD1, &sd_command_init_struct);

	delay_ms(1000);

	sd_command_init_struct.rsp_type = SD_RESPONSE_SHORT;
	sd_command_init_struct.wait_type = SD_WAIT_FOR_BUSY;
	sd_command_init_struct.cmd_index = SD_CMD_APP_SD_SET_BUSWIDTH;
	sd_command_init_struct.argument = 0x2;
	apb_sd_host_command_config(APB_SD1, &sd_command_init_struct);

	if(status == 1)
	{
		status = 0;
	}
	else if(status == 2)
	{
		status = 0;
	}

	return status;
}

//Stop SD card transfer
void sd_transfer_stop(void)
{
	/* send cmd12 */
	sd_command_init_struct.rsp_type = SD_RESPONSE_SHORT;
	sd_command_init_struct.wait_type = SD_WAIT_FOR_BUSY;
	sd_command_init_struct.transfer_direction = SD_DATA_NONE;
	sd_command_init_struct.cmd_index = SD_CMD_STOP_TRANSMISSION;
	sd_command_init_struct.argument = 0x0;
	apb_sd_host_command_config(APB_SD1, &sd_command_init_struct);

	delay_ms(500);
}

// Read SD card
uint32_t sd_read_disk(uint8_t *buf, uint32_t sector, uint8_t cnt)
{
	uint32_t read_cmd_index, count;
	uint32_t *rx_data = (uint32_t*)buf;
	long long lsector = sector;

	/* data address is in block (512 byte) units. */
	if(!SDIO_HIGH_CAPACITY_SD_CARD)
	{
		lsector <<= 9;
	}
	
	if(cnt == 1)
	{
		read_cmd_index = SD_CMD_READ_SINGLE_BLOCK;
	}
	else
	{
		read_cmd_index = SD_CMD_READ_MULT_BLOCK;
	}
	
	count = (cnt*512)/4;

	/* send cmd16, set block size */
	sd_command_init_struct.rsp_type = SD_RESPONSE_SHORT;
	sd_command_init_struct.wait_type = SD_WAIT_FOR_NO;
	sd_command_init_struct.cmd_index = SD_CMD_SET_BLOCKLEN;
	sd_command_init_struct.argument = 0x200;
	apb_sd_host_command_config(APB_SD1, &sd_command_init_struct);

	delay_ms(500);

	/* sd_block_read */
	sd_data_init_struct.timeout = DATATIMEOUT;
	sd_data_init_struct.block_size = SDIO_DATA_BLOCK_SIZE_512B;
	sd_data_init_struct.block_cnt = cnt - 1;
	apb_sd_host_data_config(APB_SD1, &sd_data_init_struct);

	sd_command_init_struct.rsp_type = SD_RESPONSE_SHORT;
	sd_command_init_struct.wait_type = SD_WAIT_FOR_NO;
	sd_command_init_struct.transfer_direction = SD_DATA_TRANSFER_TO_CONTROLLER;
	sd_command_init_struct.cmd_index = read_cmd_index;
	sd_command_init_struct.argument = lsector;
	apb_sd_host_command_config(APB_SD1, &sd_command_init_struct);

	//read
	for(int m=0;m<count;m++)
	{
		while((APB_SD1->fifo_status & 0x2));
		
		rx_data[m] = APB_SD1->rx_data;
	}

	/* sd_transfer_stop */
	sd_transfer_stop();

	return 0;
}

//Write SD card
uint32_t sd_write_disk(const uint8_t *buf, uint32_t sector, uint8_t cnt)
{
	uint32_t write_cmd_index, count;
	uint32_t *tx_data = (uint32_t*)buf;
	long long lsector = sector;

	/* data address is in block (512 byte) units. */
	if(!SDIO_HIGH_CAPACITY_SD_CARD)
	{
		lsector <<= 9;
	}

	if(cnt == 1)
	{
		write_cmd_index = SD_CMD_WRITE_SINGLE_BLOCK;
	}
	else
	{
		write_cmd_index = SD_CMD_WRITE_MULT_BLOCK;
	}
	
	count = (cnt*512)/4;

	/* send cmd16, set block size */
	sd_command_init_struct.rsp_type = SD_RESPONSE_SHORT;
	sd_command_init_struct.wait_type = SD_WAIT_FOR_NO;
	sd_command_init_struct.cmd_index = SD_CMD_SET_BLOCKLEN;
	sd_command_init_struct.argument = 0x200;
	apb_sd_host_command_config(APB_SD1, &sd_command_init_struct);

	delay_ms(500);

	sd_data_init_struct.timeout = DATATIMEOUT;
	sd_data_init_struct.block_size = SDIO_DATA_BLOCK_SIZE_512B;
	sd_data_init_struct.block_cnt = cnt - 1;
	apb_sd_host_data_config(APB_SD1, &sd_data_init_struct);

	sd_command_init_struct.rsp_type = SD_RESPONSE_SHORT;
	sd_command_init_struct.wait_type = SD_WAIT_FOR_NO;
	sd_command_init_struct.transfer_direction = SD_DATA_TRANSFER_TO_CARD;
	sd_command_init_struct.cmd_index = write_cmd_index;
	sd_command_init_struct.argument = lsector;
	apb_sd_host_command_config(APB_SD1, &sd_command_init_struct);

	//write
	for(int t=0;t<count;t++)
	{
		while(APB_SD1->fifo_status & 0x4);
		
		APB_SD1->tx_data = tx_data[t];
	}

	while(!(APB_SD1->fifo_status & 0x8));

	/* sd_transfer_stop */
	sd_transfer_stop();

	return 0;
}
