/*
 * ******************************************************************************************
 *
 * File     : apb_sd_host.h
 * Author   : GOWIN
 * Function : Definition of APB SD Host
 *
 * ******************************************************************************************
 */

#ifndef __APB_SD_HOST_H__
#define __APB_SD_HOST_H__


#ifdef __cplusplus
extern "C" {
#endif


/*****************************************************************************
 * Device Specific Peripheral Registers structures
 ****************************************************************************/
#define __RO          volatile const  /* 'read  only' permissions     */
#define __WO          volatile        /* 'write only' permissions     */
#define __RW          volatile        /* 'read/write' permissions     */


/*****************************************************************************
 * APB SD Host
 ****************************************************************************/
typedef struct
{
	__RW unsigned int  argument;           		/* 0x000 (R/W) command argument                  			       */
	__RW unsigned int  command;           		/* 0x004 (R/W) command transaction configuration             */
	__RO unsigned int  response0;            	/* 0x008 (R/ ) bits 31-0 of the response                     */
	__RO unsigned int  response1;            	/* 0x00C (R/ ) bits 63-32 of the response	                   */
	__RO unsigned int  response2;            	/* 0x010 (R/ ) bits 95-64 of the response                    */
	__RO unsigned int  response3;            	/* 0x014 (R/ ) bits 127-96 of the response                   */
	__RW unsigned int  data_timeout;         	/* 0x018 (R/W) data xfer timeout configuration               */
	__RW unsigned int  control;              	/* 0x01C (R/W) IP core control settings                      */
	__RW unsigned int  cmd_timeout;          	/* 0x020 (R/W) command xfer timeout configuration            */
	__RW unsigned int  clock_divider;        	/* 0x024 (R/W) MMC/SD interface clock divider                */
	__RW unsigned int  reset;                 /* 0x028 (R/W) software reset						                     */
	     unsigned int  RESERVED0[2];          /* 0x02C-0x30  reserved                                      */
	__RW unsigned int  cmd_event_status;     	/* 0x034 (R/W) command transaction events status / clear     */
	__RW unsigned int  cmd_event_enable;     	/* 0x038 (R/W) command transaction events enable             */
	__RW unsigned int  data_event_status;    	/* 0x03C (R/W) data transaction events status / clear        */
	__RW unsigned int  data_event_enable;    	/* 0x040 (R/W) data transaction events enable		             */
	__RW unsigned int  blkock_size;          	/* 0x044 (R/W) read / write block transfer size              */
	__RW unsigned int  blkock_count;         	/* 0x048 (R/W) read / write block count		                   */
	__WO unsigned int  tx_data;        				/* 0x04C ( /W) write data									                   */
	__RO unsigned int  rx_data;         			/* 0x050 (R/ ) read data		 							                   */
	__RO unsigned int  fifo_status;         	/* 0x054 (R/ ) fifo status		 							                 */
	     unsigned int  RESERVED1[3];          /* 0x058-0x05C reserved                                      */
} APB_SD_Host_RegDef;


/**
  * APB SD Host interrupts definition
  */
#define SD_CMDCMPL_INT                  ((unsigned char)0x01)
#define SD_CMDERROR_INT                 ((unsigned char)0x02)
#define SD_CMDTIMEOUT_INT               ((unsigned char)0x04)
#define SD_CMDCRC_INT                   ((unsigned char)0x08)
#define SD_CMDINDEX_INT                 ((unsigned char)0x10)

#define SD_DATACMPL_INT                 ((unsigned char)0x01)
#define SD_DATAERROR_INT                ((unsigned char)0x02)
#define SD_DATATIMEOUT_INT              ((unsigned char)0x04)
#define SD_DATACRC_INT                  ((unsigned char)0x08)
#define SD_DATA_WFIFO_EMPTY_INT         ((unsigned char)0x10)
#define SD_DATA_RFIFO_FULL_INT          ((unsigned char)0x20)

/**
  * APB SD Host interrupts status
  */
#define SD_CMDCMPL_FLAG                 ((unsigned char)0x01)
#define SD_CMDERROR_FLAG                ((unsigned char)0x02)
#define SD_CMDTIMEOUT_FLAG              ((unsigned char)0x04)
#define SD_CMDCRC_FLAG                  ((unsigned char)0x08)
#define SD_CMDINDEX_FLAG                ((unsigned char)0x10)

#define SD_DATACMPL_FLAG                ((unsigned char)0x01)
#define SD_DATAERROR_FLAG               ((unsigned char)0x02)
#define SD_DATATIMEOUT_FLAG             ((unsigned char)0x04)
#define SD_DATACRC_FLAG                 ((unsigned char)0x08)
#define SD_DATA_WFIFO_EMPTY_FLAG        ((unsigned char)0x10)
#define SD_DATA_RFIFO_FULL_FLAG         ((unsigned char)0x20)

/**
  * APB SD Host commands index
  */
#define SD_CMD_GO_IDLE_STATE             ((unsigned char)0)
#define SD_CMD_SEND_OP_COND              ((unsigned char)1)
#define SD_CMD_ALL_SEND_CID              ((unsigned char)2)
#define SD_CMD_SET_REL_ADDR              ((unsigned char)3)
#define SD_CMD_SET_DSR                   ((unsigned char)4)
#define SD_CMD_SDIO_SEN_OP_COND          ((unsigned char)5)
#define SD_CMD_HS_SWITCH                 ((unsigned char)6)
#define SD_CMD_SEL_DESEL_CARD            ((unsigned char)7)
#define SD_CMD_HS_SEND_IF_COND           ((unsigned char)8)
#define SD_CMD_SEND_CSD                  ((unsigned char)9)
#define SD_CMD_SEND_CID                  ((unsigned char)10)
#define SD_CMD_READ_DAT_UNTIL_STOP       ((unsigned char)11)
#define SD_CMD_STOP_TRANSMISSION         ((unsigned char)12)
#define SD_CMD_SEND_STATUS               ((unsigned char)13)
#define SD_CMD_HS_BUSTEST_READ           ((unsigned char)14)
#define SD_CMD_GO_INACTIVE_STATE         ((unsigned char)15)
#define SD_CMD_SET_BLOCKLEN              ((unsigned char)16)
#define SD_CMD_READ_SINGLE_BLOCK         ((unsigned char)17)
#define SD_CMD_READ_MULT_BLOCK           ((unsigned char)18)
#define SD_CMD_HS_BUSTEST_WRITE          ((unsigned char)19)
#define SD_CMD_WRITE_DAT_UNTIL_STOP      ((unsigned char)20)
#define SD_CMD_SET_BLOCK_COUNT           ((unsigned char)23)
#define SD_CMD_WRITE_SINGLE_BLOCK        ((unsigned char)24)
#define SD_CMD_WRITE_MULT_BLOCK          ((unsigned char)25)
#define SD_CMD_PROG_CID                  ((unsigned char)26)
#define SD_CMD_PROG_CSD                  ((unsigned char)27)
#define SD_CMD_SET_WRITE_PROT            ((unsigned char)28)
#define SD_CMD_CLR_WRITE_PROT            ((unsigned char)29)
#define SD_CMD_SEND_WRITE_PROT           ((unsigned char)30)
#define SD_CMD_SD_ERASE_GRP_START        ((unsigned char)32)
#define SD_CMD_SD_ERASE_GRP_END          ((unsigned char)33)
#define SD_CMD_ERASE_GRP_START           ((unsigned char)35)
#define SD_CMD_ERASE_GRP_END             ((unsigned char)36)
#define SD_CMD_ERASE                     ((unsigned char)38)
#define SD_CMD_FAST_IO                   ((unsigned char)39)
#define SD_CMD_GO_IRQ_STATE              ((unsigned char)40)
#define SD_CMD_LOCK_UNLOCK               ((unsigned char)42)
#define SD_CMD_APP_CMD                   ((unsigned char)55)
#define SD_CMD_GEN_CMD                   ((unsigned char)56)

#define SD_ACMD_SD_SEND_OP_COND          ((unsigned char)41)
#define SD_CMD_APP_SD_SET_BUSWIDTH       ((unsigned char)6)

/**
  * Masks for r6 response
  */

#define SD_CHECK_PATTERN                 ((unsigned int)0x000001AA)

/**
  * APB SD Host reset type
  */
typedef enum
{
  SD_NORESET,
  SD_RESET
} sd_reset_type;

/**
  * APB SD Host clock divider
  */
typedef enum
{
  SD_CLK_DIV6 = 2,
  SD_CLK_DIV8 = 3,
  SD_CLK_DIV10 = 4,
  SD_CLK_DIV12 = 5,
  SD_CLK_DIV14 = 6,
  SD_CLK_DIV16 = 7,
  SD_CLK_DIV18 = 8,
  SD_CLK_DIV20 = 9,
  SD_CLK_DIV22 = 10,
  SD_CLK_DIV24 = 11,
  SD_CLK_DIV26 = 12
} sd_clk_div_type;

/**
  * APB SD Host bus width
  */
typedef enum
{
  SD_BUS_WIDTH_D1,
  SD_BUS_WIDTH_D4
} sd_bus_width_type;

/**
  * APB SD Host response type
  */
typedef enum
{
  SD_RESPONSE_NO,
  SD_RESPONSE_SHORT,
  SD_RESPONSE_LONG
} sd_response_type;

/**
  * APB SD Host wait type
  */
typedef enum
{
  SD_WAIT_FOR_NO,
  SD_WAIT_FOR_BUSY
} sd_wait_type;

/**
  * APB SD Host CRC check type
  */
typedef enum
{
  SD_CRC_NO,
  SD_CRC_CHECK
} sd_crc_check_type;

/**
  * APB SD Host index check type
  */
typedef enum
{
  SD_INDEX_NO,
  SD_INDEX_CHECK
} sd_index_check_type;

/**
  * APB SD Host response index type
  */
typedef enum
{
  SD_RSP1_INDEX,
  SD_RSP2_INDEX,
  SD_RSP3_INDEX,
  SD_RSP4_INDEX
} sd_rsp_index_type;

/**
  * APB SD Host block size
  */
typedef enum
{
  SDIO_DATA_BLOCK_SIZE_512B = 511
} sd_block_size_type;

/**
  * APB SD Host transfer direction
  */
typedef enum
{
  SD_DATA_NONE,
  SD_DATA_TRANSFER_TO_CONTROLLER,
  SD_DATA_TRANSFER_TO_CARD
} sd_transfer_direction_type;

/**
  * APB SD Host interrupt type
  */
typedef enum
{
  SD_CMD_INT,
  SD_DATA_INT
} sd_interrupt_type;

/**
  * APB SD Host FIFO status
  */
typedef enum
{
  SD_W_FIFO_FULL,
  SD_W_FIFO_EMPTY,
  SD_R_FIFO_FULL,
  SD_R_FIFO_EMPTY
} sd_fifo_type;

/**
  * APB SD Host command structure
  */
typedef struct
{
  unsigned int                           argument;
  unsigned char                          cmd_index;
  unsigned int                           timeout;
  sd_index_check_type                    index;
  sd_crc_check_type                      crc;
  sd_response_type                       rsp_type;
  sd_wait_type                           wait_type;
  sd_transfer_direction_type             transfer_direction;
} sd_command_struct_type;

/**
  * APB SD Host data structure
  */
typedef struct
{
  unsigned int                         timeout;
  sd_block_size_type                 	 block_size;
  unsigned int                				 block_cnt;
} sd_data_struct_type;


#ifdef __cplusplus
}
#endif


#endif	/* __APB_SD_HOST_H__ */
