/*
 * ******************************************************************************************
 *
 * File		  : main.c
 * Author	  : GOWIN
 * Function	: APB SD Host demo
 *
 * ******************************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "uart.h"
#include <stdio.h>
#include "delay.h"

#include "apb_sd_host_dev.h"
#include "apb_sd_host_driver.h"


/* Defines -------------------------------------------------------------------*/

#define SD_MULT_BLOCK_TEST 1

#define START_ADDR         0x0
#define BLOCKS_NUMBER      8

#define CMDTIMEOUT         0x7FFF
#define DATATIMEOUT        0x7FFFFF

unsigned int tx_data[(BLOCKS_NUMBER*512)/4];
unsigned int rx_data[(BLOCKS_NUMBER*512)/4];

static unsigned int /*cid_table[4],*/ rca = 0; /* cid, rca */

sd_command_struct_type sd_command_init_struct;
sd_data_struct_type sd_data_init_struct;

typedef enum
{
  SD_MULT_BLOCK,
  SD_SINGLE_BLOCK
} sd_block_type;


// sd interrupt config
void sd_interrupt_config(sd_interrupt_type type, unsigned char int_opt)
{
	if(type == SD_CMD_INT)
	{
		nvic_irq_enable(IRQ_SD_CMD_SOURCE, 1, 0);
		apb_sd_host_interrupt_enable(APB_SD1, SD_CMD_INT, int_opt, 1);

	}
	else
	{
		nvic_irq_enable(IRQ_SD_DATA_SOURCE, 2, 0);
		apb_sd_host_interrupt_enable(APB_SD1, SD_DATA_INT, int_opt, 1);
	}
}

// sd initialized
unsigned char sd_init(void)
{
	unsigned char retry = 10, status;

	/* Set clock divider */
	apb_sd_host_clock_config(APB_SD1, SD_CLK_DIV24);

	/* Set data width */
	apb_sd_host_bus_width_config(APB_SD1, SD_BUS_WIDTH_D4);

	/* Set sd cmd interrupt */
	//sd_interrupt_config(SD_CMD_INT, SD_CMDCMPL_INT);
	
	/* Set sd data interrupt */
	//sd_interrupt_config(SD_DATA_INT, SD_DATACMPL_INT);

	sd_command_init_struct.timeout = CMDTIMEOUT;
	sd_command_init_struct.rsp_type = SD_RESPONSE_NO;
	sd_command_init_struct.wait_type = SD_WAIT_FOR_NO;
	sd_command_init_struct.crc = SD_CRC_CHECK;
	sd_command_init_struct.index = SD_INDEX_CHECK;
	sd_command_init_struct.cmd_index = SD_CMD_GO_IDLE_STATE;
	sd_command_init_struct.argument = 0x0;
	apb_sd_host_command_config(APB_SD1, &sd_command_init_struct);

	delay_ms(1000);

	do
	{
		sd_command_init_struct.rsp_type = SD_RESPONSE_SHORT;
		sd_command_init_struct.wait_type = SD_WAIT_FOR_BUSY;
		sd_command_init_struct.cmd_index = SD_CMD_HS_SEND_IF_COND;
		sd_command_init_struct.argument = SD_CHECK_PATTERN;
		apb_sd_host_command_config(APB_SD1, &sd_command_init_struct);

		delay_ms(1000);
		
		printf("CMD8:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP1_INDEX));
		printf("CMD8:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP2_INDEX));
		printf("CMD8:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP3_INDEX));
		printf("CMD8:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP4_INDEX));
	} while(((apb_sd_host_response_get(APB_SD1, SD_RSP1_INDEX) >> 8) != 0x000001) && retry--);

	retry = 10;

	do
	{
		sd_command_init_struct.rsp_type = SD_RESPONSE_SHORT;
		sd_command_init_struct.wait_type = SD_WAIT_FOR_BUSY;
		sd_command_init_struct.cmd_index = SD_CMD_APP_CMD;
		sd_command_init_struct.argument = 0x0;
		apb_sd_host_command_config(APB_SD1, &sd_command_init_struct);

		delay_ms(1000);
		
		printf("CMD55:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP1_INDEX));
		printf("CMD55:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP2_INDEX));
		printf("CMD55:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP3_INDEX));
		printf("CMD55:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP4_INDEX));

		sd_command_init_struct.wait_type = SD_WAIT_FOR_NO;
		sd_command_init_struct.cmd_index = SD_ACMD_SD_SEND_OP_COND;
		sd_command_init_struct.argument = 0x40ff8000;
		apb_sd_host_command_config(APB_SD1, &sd_command_init_struct);

		delay_ms(1000);
		
		printf("ACMD41:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP1_INDEX));
		printf("ACMD41:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP2_INDEX));
		printf("ACMD41:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP3_INDEX));
		printf("ACMD41:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP4_INDEX));

	} while(!((apb_sd_host_response_get(APB_SD1, SD_RSP1_INDEX) >> 24) & 0x80) && retry--);

	if((apb_sd_host_response_get(APB_SD1, SD_RSP1_INDEX) >> 24) & 0x80)
	{
		status = 1;
		
		if((apb_sd_host_response_get(APB_SD1, SD_RSP1_INDEX) >> 24) & 0x40)
		{
			status = 2;
		}
	}

	return status;
}

// sd identification
void sd_identification(void)
{
	printf("APB SD Host identification start!\r\n");

	/* send cmd2, get cid register */
	sd_command_init_struct.rsp_type = SD_RESPONSE_LONG;
	sd_command_init_struct.wait_type = SD_WAIT_FOR_BUSY;
	sd_command_init_struct.cmd_index = SD_CMD_ALL_SEND_CID;
	sd_command_init_struct.argument = 0x0;
	apb_sd_host_command_config(APB_SD1, &sd_command_init_struct);

	delay_ms(1000);
	
	printf("CMD2:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP1_INDEX));
	printf("CMD2:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP2_INDEX));
	printf("CMD2:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP3_INDEX));
	printf("CMD2:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP4_INDEX));
//  cid_table[0] = apb_sd_host_response_get(APB_SD1, SD_RSP1_INDEX);
//  cid_table[1] = apb_sd_host_response_get(APB_SD1, SD_RSP2_INDEX);
//  cid_table[2] = apb_sd_host_response_get(APB_SD1, SD_RSP3_INDEX);
//  cid_table[3] = apb_sd_host_response_get(APB_SD1, SD_RSP4_INDEX);

  /* send cmd3, get rca */
	sd_command_init_struct.rsp_type = SD_RESPONSE_SHORT;
	sd_command_init_struct.wait_type = SD_WAIT_FOR_BUSY;
	sd_command_init_struct.cmd_index = SD_CMD_SET_REL_ADDR;
	sd_command_init_struct.argument = 0x100;
	apb_sd_host_command_config(APB_SD1, &sd_command_init_struct);

	delay_ms(1000);
	
	printf("CMD3:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP1_INDEX));
	printf("CMD3:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP2_INDEX));
	printf("CMD3:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP3_INDEX));
	printf("CMD3:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP4_INDEX));
	rca = apb_sd_host_response_get(APB_SD1, SD_RSP1_INDEX);

	/* send cmd7, select card */
	sd_command_init_struct.rsp_type = SD_RESPONSE_SHORT;
	sd_command_init_struct.wait_type = SD_WAIT_FOR_BUSY;
	sd_command_init_struct.cmd_index = SD_CMD_SEL_DESEL_CARD;
	sd_command_init_struct.argument = rca & 0xFFFF0000;
	apb_sd_host_command_config(APB_SD1, &sd_command_init_struct);

	delay_ms(1000);
	
	printf("CMD7:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP1_INDEX));
	printf("CMD7:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP2_INDEX));
	printf("CMD7:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP3_INDEX));
	printf("CMD7:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP4_INDEX));

	printf("APB SD Host identification end!\r\n\r\n");

}

void sd_transfer_stop(void)
{
	/* send cmd12, stop transmission */
	sd_command_init_struct.rsp_type = SD_RESPONSE_SHORT;
	sd_command_init_struct.wait_type = SD_WAIT_FOR_BUSY;
	sd_command_init_struct.transfer_direction = SD_DATA_NONE;
	sd_command_init_struct.cmd_index = SD_CMD_STOP_TRANSMISSION;
	sd_command_init_struct.argument = 0x0;
	apb_sd_host_command_config(APB_SD1, &sd_command_init_struct);

	delay_ms(1000);
	
	printf("CMD12:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP1_INDEX));
	printf("CMD12:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP2_INDEX));
	printf("CMD12:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP3_INDEX));
	printf("CMD12:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP4_INDEX));

}

void sd_blocks_erase(uint32_t start_addr, uint32_t end_addr)
{
	printf("\r\nAPB SD Host blocks erase start!\r\n");
	
	/* sd_block_erase */
	sd_command_init_struct.rsp_type = SD_RESPONSE_SHORT;
	sd_command_init_struct.wait_type = SD_WAIT_FOR_BUSY;
	sd_command_init_struct.transfer_direction = SD_DATA_NONE;
	sd_command_init_struct.cmd_index = SD_CMD_SD_ERASE_GRP_START;
	sd_command_init_struct.argument = start_addr;
	apb_sd_host_command_config(APB_SD1, &sd_command_init_struct);

	delay_ms(1000);
	
	printf("CMD32:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP1_INDEX));
	printf("CMD32:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP2_INDEX));
	printf("CMD32:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP3_INDEX));
	printf("CMD32:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP4_INDEX));

	sd_command_init_struct.rsp_type = SD_RESPONSE_SHORT;
	sd_command_init_struct.wait_type = SD_WAIT_FOR_BUSY;
	sd_command_init_struct.transfer_direction = SD_DATA_NONE;
	sd_command_init_struct.cmd_index = SD_CMD_SD_ERASE_GRP_END;
	sd_command_init_struct.argument = end_addr;
	apb_sd_host_command_config(APB_SD1, &sd_command_init_struct);

	delay_ms(1000);
	
	printf("CMD33:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP1_INDEX));
	printf("CMD33:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP2_INDEX));
	printf("CMD33:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP3_INDEX));
	printf("CMD33:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP4_INDEX));

	sd_command_init_struct.rsp_type = SD_RESPONSE_SHORT;
	sd_command_init_struct.wait_type = SD_WAIT_FOR_BUSY;
	sd_command_init_struct.transfer_direction = SD_DATA_NONE;
	sd_command_init_struct.cmd_index = SD_CMD_ERASE;
	sd_command_init_struct.argument = 0x0;
	apb_sd_host_command_config(APB_SD1, &sd_command_init_struct);

	delay_ms(1000);
	
	printf("CMD38:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP1_INDEX));
	printf("CMD38:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP2_INDEX));
	printf("CMD38:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP3_INDEX));
	printf("CMD38:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP4_INDEX));

	printf("APB SD Host blocks erase end!\r\n\r\n");
}

void sd_test(sd_block_type block_type, uint32_t start_addr, uint32_t blocks_num)
{
	uint32_t count, block_cnt, write_cmd_index, read_cmd_index, end_addr;

	/* According to different block type tests, the corresponding parameters are initialized */
	if(block_type == SD_MULT_BLOCK)
	{
		write_cmd_index = SD_CMD_WRITE_MULT_BLOCK;
		read_cmd_index = SD_CMD_READ_MULT_BLOCK;
	}
	else
	{
		write_cmd_index = SD_CMD_WRITE_SINGLE_BLOCK;
		read_cmd_index = SD_CMD_READ_SINGLE_BLOCK;
	}
	count = (blocks_num*512)/4;
	block_cnt = blocks_num - 1;
	end_addr = start_addr + blocks_num;

	/* set data width */
	sd_command_init_struct.rsp_type = SD_RESPONSE_SHORT;
	sd_command_init_struct.wait_type = SD_WAIT_FOR_BUSY;
	sd_command_init_struct.cmd_index = SD_CMD_APP_CMD;
	sd_command_init_struct.argument = rca & 0xFFFF0000;			// SanDisk
	apb_sd_host_command_config(APB_SD1, &sd_command_init_struct);

	delay_ms(1000);
	
	printf("CMD55:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP1_INDEX));
	printf("CMD55:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP2_INDEX));
	printf("CMD55:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP3_INDEX));
	printf("CMD55:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP4_INDEX));

	sd_command_init_struct.rsp_type = SD_RESPONSE_SHORT;
	sd_command_init_struct.wait_type = SD_WAIT_FOR_BUSY;
	sd_command_init_struct.cmd_index = SD_CMD_APP_SD_SET_BUSWIDTH;
	sd_command_init_struct.argument = 0x2;
	apb_sd_host_command_config(APB_SD1, &sd_command_init_struct);

	delay_ms(1000);
	
	printf("ACMD6:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP1_INDEX));
	printf("ACMD6:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP2_INDEX));
	printf("ACMD6:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP3_INDEX));
	printf("ACMD6:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP4_INDEX));

	/* send cmd16, set block size */
	sd_command_init_struct.rsp_type = SD_RESPONSE_SHORT;
	sd_command_init_struct.wait_type = SD_WAIT_FOR_BUSY;
	sd_command_init_struct.cmd_index = SD_CMD_SET_BLOCKLEN;
	sd_command_init_struct.argument = 0x200;
	apb_sd_host_command_config(APB_SD1, &sd_command_init_struct);

	delay_ms(1000);
	
	printf("CMD16:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP1_INDEX));
	printf("CMD16:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP2_INDEX));
	printf("CMD16:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP3_INDEX));
	printf("CMD16:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP4_INDEX));

	/* erase blocks */
	sd_blocks_erase(start_addr, end_addr);

	/* read data */
	sd_data_init_struct.timeout = DATATIMEOUT;
	sd_data_init_struct.block_size = SDIO_DATA_BLOCK_SIZE_512B;
	sd_data_init_struct.block_cnt = block_cnt;
	apb_sd_host_data_config(APB_SD1, &sd_data_init_struct);

	sd_command_init_struct.rsp_type = SD_RESPONSE_SHORT;
	sd_command_init_struct.wait_type = SD_WAIT_FOR_BUSY;
	sd_command_init_struct.transfer_direction = SD_DATA_TRANSFER_TO_CONTROLLER;
	sd_command_init_struct.cmd_index = read_cmd_index;
	sd_command_init_struct.argument = start_addr;
	apb_sd_host_command_config(APB_SD1, &sd_command_init_struct);

	/* read */
	for(int m=0;m<count;m++)
	{
		while(apb_sd_host_fifo_status_get(APB_SD1, SD_R_FIFO_EMPTY));
		
		apb_sd_host_data_read(APB_SD1, rx_data+m);
	}

	/* Stop transfer */
	sd_transfer_stop();

	for(int n=0;n<count;n++)
	{
		printf("SD_data[%d] = 0x%08x\r\n", n, rx_data[n]);
	}

	for(int k=0;k<count;k++)
	{
		tx_data[k]=k+1;
	}

	/* Write data */
	sd_data_init_struct.timeout = DATATIMEOUT;
	sd_data_init_struct.block_size = SDIO_DATA_BLOCK_SIZE_512B;
	sd_data_init_struct.block_cnt = block_cnt;
	apb_sd_host_data_config(APB_SD1, &sd_data_init_struct);

	sd_command_init_struct.rsp_type = SD_RESPONSE_SHORT;
	sd_command_init_struct.wait_type = SD_WAIT_FOR_BUSY;
	sd_command_init_struct.transfer_direction = SD_DATA_TRANSFER_TO_CARD;
	sd_command_init_struct.cmd_index = write_cmd_index;
	sd_command_init_struct.argument = start_addr;
	apb_sd_host_command_config(APB_SD1, &sd_command_init_struct);

	printf("CMD25:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP1_INDEX));
	printf("CMD25:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP2_INDEX));
	printf("CMD25:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP3_INDEX));
	printf("CMD25:0x%08x \r\n", apb_sd_host_response_get(APB_SD1, SD_RSP4_INDEX));

	/* write */
	for(int t=0;t<count;t++)
	{
		while(apb_sd_host_fifo_status_get(APB_SD1, SD_W_FIFO_FULL));
		
		apb_sd_host_data_write(APB_SD1, tx_data+t);
	}

	while(!(apb_sd_host_fifo_status_get(APB_SD1, SD_W_FIFO_EMPTY)));

	/* Stop transfer */
	sd_transfer_stop();

	/* Read data */
	sd_data_init_struct.timeout = DATATIMEOUT;
	sd_data_init_struct.block_size = SDIO_DATA_BLOCK_SIZE_512B;
	sd_data_init_struct.block_cnt = block_cnt;
	apb_sd_host_data_config(APB_SD1, &sd_data_init_struct);

	sd_command_init_struct.rsp_type = SD_RESPONSE_SHORT;
	sd_command_init_struct.wait_type = SD_WAIT_FOR_BUSY;
	sd_command_init_struct.transfer_direction = SD_DATA_TRANSFER_TO_CONTROLLER;
	sd_command_init_struct.cmd_index = read_cmd_index;
	sd_command_init_struct.argument = start_addr;
	apb_sd_host_command_config(APB_SD1, &sd_command_init_struct);

	/* read */
	for(int m=0;m<count;m++)
	{
		while(apb_sd_host_fifo_status_get(APB_SD1, SD_R_FIFO_EMPTY));
		
		apb_sd_host_data_read(APB_SD1, rx_data+m);
	}

	/* Stop transfer */
	sd_transfer_stop();

	for(int n=0;n<count;n++)
	{
		printf("rx_data[%d] = 0x%08x\r\n", n, rx_data[n]);
		
		if(rx_data[n] != tx_data[n])
		{
			printf("\r\nFail!\r\n");
			break;
		}

	}

	printf("\r\nPass!\r\n");
}

// APB SD Host command interrupt handler
void sd_cmd_irq_handler(void)
{
	// Disable interrupt
	nvic_irq_disable(IRQ_SD_CMD_SOURCE);
	
	if(apb_sd_host_interrupt_status_get(APB_SD1, SD_CMD_INT, SD_CMDCMPL_FLAG) == 1)
	{
		apb_sd_host_interrupt_status_clear(APB_SD1, SD_CMD_INT, SD_CMDCMPL_FLAG);
		
		//printf("SD_CMDCMPL_FLAG!\r\n");
		//printf("cmd_event_status:0x%08x\r\n", APB_SD1->cmd_event_status);
	}
	else if(apb_sd_host_interrupt_status_get(APB_SD1, SD_CMD_INT, SD_CMDERROR_FLAG) == 1)
	{
		apb_sd_host_interrupt_status_clear(APB_SD1, SD_CMD_INT, SD_CMDERROR_FLAG);
		
		printf("SD_CMDERROR_FLAG!\r\n");
	}
	else if(apb_sd_host_interrupt_status_get(APB_SD1, SD_CMD_INT, SD_CMDTIMEOUT_FLAG) == 1)
	{
		apb_sd_host_interrupt_status_clear(APB_SD1, SD_CMD_INT, SD_CMDTIMEOUT_FLAG);
		
		printf("SD_CMDTIMEOUT_FLAG!\r\n");
	}
	else if(apb_sd_host_interrupt_status_get(APB_SD1, SD_CMD_INT, SD_CMDCRC_FLAG) == 1)
	{
		apb_sd_host_interrupt_status_clear(APB_SD1, SD_CMD_INT, SD_CMDCRC_FLAG);
		
		printf("SD_CMDCRC_FLAG!\r\n");
	}
	else if(apb_sd_host_interrupt_status_get(APB_SD1, SD_CMD_INT, SD_CMDINDEX_FLAG) == 1)
	{
		apb_sd_host_interrupt_status_clear(APB_SD1, SD_CMD_INT, SD_CMDINDEX_FLAG);
		
		printf("SD_CMDINDEX_FLAG!\r\n");
		printf("cmd_event_status:0x%08x\r\n", APB_SD1->cmd_event_status);
	}
	
	// Enable interrupt
	nvic_irq_enable(IRQ_SD_CMD_SOURCE, 1, 0);
}

// APB SD Host data interrupt handler
void sd_data_irq_handler(void)
{
	// Disable interrupt
	nvic_irq_disable(IRQ_SD_DATA_SOURCE);
	
	if(apb_sd_host_interrupt_status_get(APB_SD1, SD_DATA_INT, SD_DATACMPL_FLAG) == 1)
	{
		apb_sd_host_interrupt_status_clear(APB_SD1, SD_DATA_INT, SD_DATACMPL_FLAG);
		
		printf("SD_DATACMPL_FLAG!\r\n");
		printf("data_event_status:0x%08x\r\n", APB_SD1->data_event_status);
	}
	else if(apb_sd_host_interrupt_status_get(APB_SD1, SD_DATA_INT, SD_DATAERROR_FLAG) == 1)
	{
		apb_sd_host_interrupt_status_clear(APB_SD1, SD_DATA_INT, SD_DATAERROR_FLAG);
		
		printf("SD_DATAERROR_FLAG!\r\n");
	}
	else if(apb_sd_host_interrupt_status_get(APB_SD1, SD_DATA_INT, SD_DATATIMEOUT_FLAG) == 1)
	{
		apb_sd_host_interrupt_status_clear(APB_SD1, SD_DATA_INT, SD_DATATIMEOUT_FLAG);
		
		printf("SD_DATATIMEOUT_FLAG!\r\n");
	}
	else if(apb_sd_host_interrupt_status_get(APB_SD1, SD_DATA_INT, SD_DATACRC_FLAG) == 1)
	{
		apb_sd_host_interrupt_status_clear(APB_SD1, SD_DATA_INT, SD_DATACRC_FLAG);
		
		printf("SD_DATACRC_FLAG!\r\n");
		printf("data_event_status:0x%08x\r\n", APB_SD1->data_event_status);
	}
	else if(apb_sd_host_interrupt_status_get(APB_SD1, SD_DATA_INT, SD_DATA_WFIFO_EMPTY_FLAG) == 1)
	{
		apb_sd_host_interrupt_status_clear(APB_SD1, SD_DATA_INT, SD_DATA_WFIFO_EMPTY_FLAG);
		
		printf("SD_DATA_WFIFO_EMPTY_FLAG!\r\n");
		printf("data_event_status:0x%08x\r\n", APB_SD1->data_event_status);
	}
	else if(apb_sd_host_interrupt_status_get(APB_SD1, SD_DATA_INT, SD_DATA_RFIFO_FULL_FLAG) == 1)
	{
		apb_sd_host_interrupt_status_clear(APB_SD1, SD_DATA_INT, SD_DATA_RFIFO_FULL_FLAG);
		
		printf("SD_DATA_RFIFO_FULL_FLAG!\r\n");
		printf("data_event_status:0x%08x\r\n", APB_SD1->data_event_status);
	}
	
	nvic_irq_enable(IRQ_SD_DATA_SOURCE, 2, 0);
}

//Application entry function
int main(void)
{
  SystemInit();  								//Initializes system clock
  uart_init(UART0,							//Initializes UART0
	          115200,							//Baudrate
	          1,									//Tx
	          1,									//Rx
	          0,									//Tx interrupt
	          0,									//Rx interrupt
	          0,									//Tx overflow interrupt
	          0);									//Rx overflow interrupt
	delay_init();									//Initializes delay function
	
	uint8_t status;
	
	printf("\r\nIt's an APB SD Host demo.\r\n\r\n");
	
	apb_sd_host_reset(APB_SD1, SD_RESET);

	/***********sd initialization and identification stage************/
	status = sd_init();
	if(status == 1)
	{
		printf("APB SD Host is initialized successfully!\r\n\r\n");
	}
	else if(status == 2)
	{
		printf("APB SD Host is initialized successfully!\r\n\r\n");
	}
	else
	{
		printf("APB SD Host is initialized error!\r\n\r\n");
	}

	sd_identification();
	
	/************************data transfer stage**********************/
#if SD_MULT_BLOCK_TEST
		sd_test(SD_MULT_BLOCK, START_ADDR, BLOCKS_NUMBER);
#else
		sd_test(SD_SINGLE_BLOCK, START_ADDR, 1);
#endif

	return 0;
}
